/* 
groundwater_init.c
read groundwater depth information if it is available

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v6.2.
Copyright 2020, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "ini.h"
#include "bgc_struct.h"
#include "pointbgc_struct.h"
#include "pointbgc_func.h"
#include "bgc_constants.h"



int groundwater_init(soilprop_struct* sprop, control_struct* ctrl)
{
	int errorCode=0;
	file GWD_file;	

	int dataread;
	int ndata = 0;

	int p1,p2,p3, maxGWD_num;
	double p4;
	char tempvar;

	int* GWDyear_array;			
	int* GWDmonth_array;						
	int* GWDday_array;							

	maxGWD_num=ctrl->simyears* nDAYS_OF_YEAR;
	/* ------------------------------------------------------ */
	/* using varying whole plant mortality values */
	
	/* GWD flag: constans or varying GWD from file */
	ctrl->GWD_flag = 0;
	if (!errorCode)
	{
		if (ctrl->spinup == 0)   /* normal run */
		{
			strcpy(GWD_file.name, "groundwater_normal.txt");
			if (!file_open(&GWD_file,'j',1)) ctrl->GWD_flag = 1;
		}
		else                     /* spinup and transient run */        
		{ 	
			strcpy(GWD_file.name, "groundwater_spinup.txt");
			if (!file_open(&GWD_file,'j',1)) ctrl->GWD_flag = 1;	
		}
	}


	if (!errorCode && ctrl->GWD_flag) 
	{		
		/* allocate space for the annual GWD array */
		sprop->GWD_array = (double*) malloc(ctrl->simyears * nDAYS_OF_YEAR * sizeof(double));
		if (!sprop->GWD_array)
		{
			printf("ERROR allocating for GWD array, groundwater_init()\n");
			errorCode=219;
		}

		/* allocate space for the temporary MGM array */
		GWDyear_array    = (int*) malloc(maxGWD_num*sizeof(int));  
		GWDmonth_array   = (int*) malloc(maxGWD_num*sizeof(int)); 
		GWDday_array     = (int*) malloc(maxGWD_num*sizeof(int)); 

		/* initialization with default value */
		for (ndata = 0; ndata < maxGWD_num; ndata++) sprop->GWD_array[ndata] = 100;
		
		ndata=0;
		while (!errorCode && !(dataread = scan_array (GWD_file, &p1, 'i', 0, 0)))
		{
			dataread = fscanf(GWD_file.ptr, "%c%d%c%d%lf%*[^\n]", &tempvar,&p2,&tempvar,&p3,&p4);
				
			if (p1 >= ctrl->simstartyear && p1 < ctrl->simstartyear + ctrl->simyears)
			{
				GWDyear_array[ndata]     = p1;
				GWDmonth_array[ndata]    = p2;
				GWDday_array[ndata]      = p3;
				sprop->GWD_array[ndata]  = p4;

				if (sprop->GWD_array[ndata] < 0.0)
				{
					/* temporary solution (balus): negative GWD data -> other method */ 
					ctrl->GWD_flag = 2;
					sprop->GWD_array[ndata] = -1 * sprop->GWD_array[ndata];
					//printf("ERROR in groundwater_init(): negative groundwater data OR too short GW-file\n");
					//errorCode=21902;
				}
			
				ndata += 1;
			}
		}
		/* read year and GWD for each simday in each simyear */
		
		free(GWDyear_array);	
		free(GWDmonth_array);	
		free(GWDday_array);	

		fclose(GWD_file.ptr);
	}	
	else /* if no changing data constant sitec parameter are used */
	{
		sprop->GWD_array = 0;
	}	
	



	/* -------------------------------------------*/

		
	return (errorCode);
}
